%{
#include <stdio.h>
extern int yylex();
int yyerror(const char *s);
%}
%union {int n; char *s;}
%token <s> NAME
%token <n> NUM IF WHILE DO
%token INT ELSE HALT OUT
%type <n> if
%left OR
%left AND
%left '|'
%left '^'
%left '&'
%left EQ NE
%left GE LE '<' '>'
%left SHL SHR
%left '+' '-'
%left '*'
%right '!' '~' NEG
%%
stmts:|stmts stmt
;
stmt: intdef|ifelse|while|do|halt|out|assign
;
intdef: INT intlist ';'
;
intlist: integer
| intlist ',' integer
;
integer: NAME {printf("%s:\t0\n",$1);free($1);}
| NAME '=' NUM {printf("%s:\t%d\n",$1,$3);free($1);}
| NAME '=' '-' NUM {printf("%s:\t%d\n",$1,-$4);free($1);}
;
ifelse: if {printf("L%dF:\n",$1);}
| if {printf("\tJMP L%dT\nL%dF:\n",$1,$1);} ELSE '{' stmts '}' {printf("L%dT:\n",$1);}
;
if: IF '(' expr ')' {printf("\tJZ L%dF\n",$1);} '{' stmts '}' {$$=$1;}
;
while: WHILE {printf("L%dT:\n",$1);} '(' expr ')' {printf("\tJZ L%dF\n",$1);} '{' stmts '}' {printf("\tJMP L%dT\nL%dF:\n",$1,$1);}
;
do: DO {printf("L%dT:\n",$1);} '{' stmts '}' WHILE '(' expr ')' ';' {printf("\tJNZ L%dT\n",$1);}
;
halt : HALT ';' {printf("\tHALT\n");}
;
out: OUT '(' expr ')' ';' {printf("\tOUT\n");}
;
assign: NAME '=' expr ';' {printf("\tPOP %s\n",$1);free($1);}
;
expr: NAME {printf("\tPUSH %s\n",$1);free($1);}
| NUM {printf("\tPUSHI %d\n",$1);}
| '!' expr {printf("\tNOT\n");}
| '~' expr {printf("\tBNOT\n");}
| '-' expr %prec NEG {printf("\tNEG\n");}
| expr '+' expr {printf("\tADD\n");}
| expr '-' expr {printf("\tSUB\n");}
| expr '*' expr {printf("\tMUL\n");}
| expr AND expr {printf("\tAND\n");}
| expr OR expr {printf("\tOR\n");}
| expr '&' expr {printf("\tBAND\n");}
| expr '|' expr {printf("\tBOR\n");}
| expr '^' expr {printf("\tBXOR\n");}
| expr SHL expr {printf("\tSHL\n");}
| expr SHR expr {printf("\tSHR\n");}
| expr EQ expr {printf("\tEQ\n");}
| expr NE expr {printf("\tNE\n");}
| expr GE expr {printf("\tGE\n");}
| expr LE expr {printf("\tLE\n");}
| expr '>' expr {printf("\tGT\n");}
| expr '<' expr {printf("\tLT\n");}
| '(' expr ')'
;
%%
int yyerror(const char *s){printf("%s\n",s);}
int main(){yyparse();}
